@@ -3,21 +3,20 @@ import logging
 import os
 import re
 from subprocess import CalledProcessError
+import textwrap
 
+from .errors import AnsibleCliError, CliError
+from .installation import build_tmp_install_script, execute_install_script
 from .system import exec_shell_live, exec_shell
 
 
-class AnsibleCliError(Exception):
-    pass
-
-
 class AnsibleCli:
 
     def __init__(self, dir, bin_path=None):
         self.dir = dir
         # Ansible supported versions interval
         self.min_version = (0, 0, 0)
-        self.max_version = (2, 10, 6)
+        self.max_version = (2, 10, 7)
         # Path to look up for executable
         self.bin_path = None
         # Force Ansible binary path if bin_path exists and contains
@@ -44,7 +43,7 @@ class AnsibleCli:
             logging.error("Failed to execute the command: %s", e.cmd)
             logging.error("Return code is: %s", e.returncode)
             logging.error("Output: %s", e.output)
-            raise Exception(
+            raise CliError(
                 "Ansible executable seems to be missing. Please install it or "
                 "check your PATH variable"
             )
@@ -59,7 +58,7 @@ class AnsibleCli:
                 break
 
         if version is None:
-            raise Exception("Unable to parse Ansible version")
+            raise CliError("Unable to parse Ansible version")
 
         logging.info("Ansible version: %s", '.'.join(map(str, version)))
 
@@ -74,7 +73,7 @@ class AnsibleCli:
                 break
 
             if version[i] not in list(range(min, max + 1)):
-                raise Exception(
+                raise CliError(
                     ("Ansible version %s not supported, must be between %s and"
                      " %s") % (
                         '.'.join(map(str, version)),
@@ -117,11 +116,10 @@ class AnsibleCli:
             )
 
     def run_playbook(
-        self, ssh_user, ssh_priv_key, inventory, playbook, extra_vars
+        self, cloud, ssh_user, ssh_priv_key, inventory, playbook, extra_vars
     ):
         try:
-            rc = exec_shell_live(
-                [
+            command = [
                     self.bin("ansible-playbook"),
                     playbook,
                     "--ssh-common-args='-o StrictHostKeyChecking=no'",
@@ -129,9 +127,11 @@ class AnsibleCli:
                     "-u", ssh_user,
                     "--private-key", ssh_priv_key,
                     "-e", "'%s'" % extra_vars
-                ],
-                cwd=self.dir
-            )
+                ]
+            if cloud in ['aws-rds', 'aws-rds-aurora']:
+                command.append('--limit')
+                command.append('!primary')
+            rc = exec_shell_live(command, cwd=self.dir)
             if rc != 0:
                 raise Exception("Return code not 0")
         except Exception as e:
@@ -167,3 +167,41 @@ class AnsibleCli:
                 "Failed to execute the following command, please check the "
                 "logs for details: %s" % e.cmd
             )
+
+    def install(self, installation_path):
+        """
+        Ansible installation
+        """
+        # Installation bash script content
+        installation_script = textwrap.dedent("""
+            #!/bin/bash
+            set -eu
+
+            mkdir -p {path}/ansible/{version}
+            python3 -m venv {path}/ansible/{version}
+            sed -i.bak 's/$1/${{1:-}}/' {path}/ansible/{version}/bin/activate
+            source {path}/ansible/{version}/bin/activate
+            python3 -m pip install "cryptography==3.3.2"
+            python3 -m pip install "ansible-base=={version}"
+            python3 -m pip install "ansible=={version}"
+            deactivate
+            rm -f {path}/bin/ansible
+            ln -sf {path}/ansible/{version}/bin/ansible {path}/bin/.
+            rm -f {path}/bin/ansible-galaxy
+            ln -sf {path}/ansible/{version}/bin/ansible-galaxy {path}/bin/.
+            rm -f {path}/bin/ansible-playbook
+            ln -sf {path}/ansible/{version}/bin/ansible-playbook {path}/bin/.
+            rm -f {path}/bin/ansible-inventory
+            ln -sf {path}/ansible/{version}/bin/ansible-inventory {path}/bin/.
+        """)
+
+        # Generate the installation script as an executable tempfile
+        script_name = build_tmp_install_script(
+            installation_script.format(
+                path=installation_path,
+                version='.'.join(str(i) for i in self.max_version)
+            )
+        )
+
+        # Execute the installation script
+        execute_install_script(script_name)
